unit AcCommon;

interface

uses
  Windows, SysUtils, Variants, Classes, Graphics;

const
  acPaperSpace = $00000000;
  acModelSpace = $00000001;
  
type
  TLayerState = (lsLayerOn, lsFreeze, lsLock, lsPlottable);
  TLayerStates = set of TLayerState;

  PAcadLayerRec = ^TAcadLayerRec;
  TAcadLayerRec = record
    lyName: String[64];
    lyColor: Integer;
    lyState: Integer;
  end;

  function EnumAutoCADLayers(Items: TStrings): Integer;

implementation

uses
  AcDvbCommon;

function AcadPoint(X, Y: Double; Z: Double = 0): OleVariant;
begin
  Result:= VarArrayCreate([0, 2], varDouble);
  Result[0]:= X;
  Result[1]:= Y;
  Result[2]:= Z;
end;

function AcadColorToTColor(acColor: Word): TColor; forward;

function EnumAutoCADLayers(Items: TStrings): Integer;
var
  Layers, Layer: OleVariant;
  ActiveLayerName: String;
  i, LayersCount, Tmp: Integer;
  States: TLayerStates;
  LayerRec: PAcadLayerRec;
begin
  Result:= -1;
  if not VarIsClear(Acad) then
  begin
    Layers:= Acad.ActiveDocument.Layers;
    LayersCount:= Layers.Count;
    with Items do
    begin
      BeginUpdate;
      try
        Clear;
        ActiveLayerName:= Acad.ActiveDocument.ActiveLayer.Name;

        for i:= 0 to LayersCount - 1 do
        begin
          Layer:= Layers.Item(i);

          States:= [];
          if Layer.LayerOn then States:= States + [lsLayerOn];
          if Layer.Freeze then States:= States + [lsFreeze];
          if Layer.Lock then States:= States + [lsLock];
          if Layer.Plottable then States:= States + [lsPlottable];
          System.Move(States, Tmp, 1);

          New(LayerRec);
          LayerRec^.lyName:= Layer.Name;
          LayerRec^.lyState:= Tmp;
          LayerRec^.lyColor:= AcadColorToTColor(Integer(Layers.Item(i).Color));

          AddObject(LayerRec^.lyName, TObject(LayerRec));
          if AnsiCompareText(LayerRec^.lyName, ActiveLayerName) = 0 then
            Result:= i;
        end;
      finally
        EndUpdate;
      end;
    end;
  end;
end;

function AcadColorToTColor(acColor: Word): TColor;
begin
  Result:= RGB(0, 0, 0);
  case acColor of
    0  : Result:= RGB(0, 0, 0);
    1  : Result:= RGB(255, 0, 0);
    2  : Result:= RGB(255, 255, 0);
    3  : Result:= RGB(0, 255, 0);
    4  : Result:= RGB(0, 255, 255);
    5  : Result:= RGB(0, 0, 255);
    6  : Result:= RGB(255, 0, 255);
    7  : Result:= RGB(255, 255, 255);
    8  : Result:= RGB(65, 65, 65);
    9  : Result:= RGB(128, 128, 128);
    10 : Result:= RGB(255, 0, 0);
    11 : Result:= RGB(255, 170, 170);
    12 : Result:= RGB(189, 0, 0);
    13 : Result:= RGB(189, 126, 126);
    14 : Result:= RGB(129, 0, 0);
    15 : Result:= RGB(129, 86, 86);
    16 : Result:= RGB(104, 0, 0);
    17 : Result:= RGB(104, 69, 69);
    18 : Result:= RGB(79, 0, 0);
    19 : Result:= RGB(79, 53, 53);
    20 : Result:= RGB(255, 63, 0);
    21 : Result:= RGB(255, 191, 170);
    22 : Result:= RGB(189, 46, 0);
    23 : Result:= RGB(189, 141, 126);
    24 : Result:= RGB(129, 31, 0);
    25 : Result:= RGB(129, 96, 86);
    26 : Result:= RGB(104, 25, 0);
    27 : Result:= RGB(104, 78, 69);
    28 : Result:= RGB(79, 19, 0);
    29 : Result:= RGB(79, 59, 53);
    30 : Result:= RGB(255, 127, 0);
    31 : Result:= RGB(255, 212, 170);
    32 : Result:= RGB(189, 94, 0);
    33 : Result:= RGB(89, 157, 126);
    34 : Result:= RGB(129, 64, 0);
    35 : Result:= RGB(129, 107, 86);
    36 : Result:= RGB(104, 52, 0);
    37 : Result:= RGB(104, 86, 69);
    38 : Result:= RGB(79, 39, 0);
    39 : Result:= RGB(79, 66, 53);
    40 : Result:= RGB(255, 191, 0);
    41 : Result:= RGB(255, 234, 170);
    42 : Result:= RGB(189, 141, 0);
    43 : Result:= RGB(189, 173, 126);
    44 : Result:= RGB(129, 96, 0);
    45 : Result:= RGB(129, 118, 86);
    46 : Result:= RGB(104, 78, 0);
    47 : Result:= RGB(104, 95, 69);
    48 : Result:= RGB(79, 59, 0);
    49 : Result:= RGB(79, 73, 53);
    50 : Result:= RGB(255, 255, 0);
    51 : Result:= RGB(255, 255, 170);
    52 : Result:= RGB(189, 189, 0);
    53 : Result:= RGB(189, 189, 126);
    54 : Result:= RGB(129, 129, 0);
    55 : Result:= RGB(129, 129, 86);
    56 : Result:= RGB(104, 104, 0);
    57 : Result:= RGB(104, 104, 69);
    58 : Result:= RGB(79, 79, 0);
    59 : Result:= RGB(79, 79, 53);
    60 : Result:= RGB(191, 255, 0);
    61 : Result:= RGB(234, 255, 170);
    62 : Result:= RGB(141, 189, 0);
    63 : Result:= RGB(173, 189, 126);
    64 : Result:= RGB(96, 129, 0);
    65 : Result:= RGB(118, 129, 86);
    66 : Result:= RGB(78, 104, 0);
    67 : Result:= RGB(95, 104, 69);
    68 : Result:= RGB(59, 79, 0);
    69 : Result:= RGB(73, 79, 53);
    70 : Result:= RGB(127, 255, 0);
    71 : Result:= RGB(212, 255, 170);
    72 : Result:= RGB(94, 189, 0);
    73 : Result:= RGB(157, 189, 126);
    74 : Result:= RGB(64, 129, 0);
    75 : Result:= RGB(107, 129, 86);
    76 : Result:= RGB(52, 104, 0);
    77 : Result:= RGB(86, 104, 69);
    78 : Result:= RGB(39, 79, 0);
    79 : Result:= RGB(66, 79, 53);
    80 : Result:= RGB(63, 255, 0);
    81 : Result:= RGB(191, 255, 170);
    82 : Result:= RGB(46, 189, 0);
    83 : Result:= RGB(141, 189, 126);
    84 : Result:= RGB(31, 129, 0);
    85 : Result:= RGB(96, 129, 86);
    86 : Result:= RGB(25, 104, 0);
    87 : Result:= RGB(78, 104, 69);
    88 : Result:= RGB(19, 79, 0);
    89 : Result:= RGB(59, 79, 53);
    90 : Result:= RGB(0, 255, 0);
    91 : Result:= RGB(170, 255, 170);
    92 : Result:= RGB(0, 189, 0);
    93 : Result:= RGB(126, 189, 126);
    94 : Result:= RGB(0, 129, 0);
    95 : Result:= RGB(86, 129, 86);
    96 : Result:= RGB(0, 104, 0);
    97 : Result:= RGB(69, 104, 69);
    98 : Result:= RGB(0, 79, 0);
    99 : Result:= RGB(53, 79, 53);
    100: Result:= RGB(0, 255, 63);
    101: Result:= RGB(170, 255, 191);
    102: Result:= RGB(0, 189, 46);
    103: Result:= RGB(126, 189, 141);
    104: Result:= RGB(0, 129, 31);
    105: Result:= RGB(86, 129, 96);
    106: Result:= RGB(0, 104, 25);
    107: Result:= RGB(69, 104, 78);
    108: Result:= RGB(0, 79, 19);
    109: Result:= RGB(53, 79, 59);
    110: Result:= RGB(0, 255, 127);
    111: Result:= RGB(170, 255, 212);
    112: Result:= RGB(0, 189, 94);
    113: Result:= RGB(126, 189, 157);
    114: Result:= RGB(0, 129, 64);
    115: Result:= RGB(86, 129, 107);
    116: Result:= RGB(0, 104, 52);
    117: Result:= RGB(69, 104, 86);
    118: Result:= RGB(0, 79, 39);
    119: Result:= RGB(53, 79, 66);
    120: Result:= RGB(0, 255, 191);
    121: Result:= RGB(170, 255, 234);
    122: Result:= RGB(0, 189, 141);
    123: Result:= RGB(126, 189, 173);
    124: Result:= RGB(0, 129, 96);
    125: Result:= RGB(86, 129, 118);
    126: Result:= RGB(0, 104, 78);
    127: Result:= RGB(69, 104, 95);
    128: Result:= RGB(0, 79, 59);
    129: Result:= RGB(53, 79, 73);
    130: Result:= RGB(0, 255, 255);
    131: Result:= RGB(170, 255, 255);
    132: Result:= RGB(0, 189, 189);
    133: Result:= RGB(126, 189, 189);
    134: Result:= RGB(0, 129, 129);
    135: Result:= RGB(86, 129, 129);
    136: Result:= RGB(0, 104, 104);
    137: Result:= RGB(69, 104, 104);
    138: Result:= RGB(0, 79, 79);
    139: Result:= RGB(53, 79, 79);
    140: Result:= RGB(0, 191, 255);
    141: Result:= RGB(170, 234, 255);
    142: Result:= RGB(0, 141, 189);
    143: Result:= RGB(126, 173, 189);
    144: Result:= RGB(0, 96, 129);
    145: Result:= RGB(86, 118, 129);
    146: Result:= RGB(0, 78, 104);
    147: Result:= RGB(69, 95, 104);
    148: Result:= RGB(0, 59, 79);
    149: Result:= RGB(53, 73, 79);
    150: Result:= RGB(0, 127, 255);
    151: Result:= RGB(170, 212, 255);
    152: Result:= RGB(0, 94, 189);
    153: Result:= RGB(126, 157, 189);
    154: Result:= RGB(0, 64, 129);
    155: Result:= RGB(86, 107, 129);
    156: Result:= RGB(0, 52, 104);
    157: Result:= RGB(69, 86, 104);
    158: Result:= RGB(0, 39, 79);
    159: Result:= RGB(53, 66, 79);
    160: Result:= RGB(0, 63, 255);
    161: Result:= RGB(170, 191, 255);
    162: Result:= RGB(0, 46, 189);
    163: Result:= RGB(126, 141, 189);
    164: Result:= RGB(0, 31, 129);
    165: Result:= RGB(86, 96, 129);
    166: Result:= RGB(0, 25, 104);
    167: Result:= RGB(69, 78, 104);
    168: Result:= RGB(0, 19, 79);
    169: Result:= RGB(53, 59, 79);
    170: Result:= RGB(0, 0, 255);
    171: Result:= RGB(170, 170, 255);
    172: Result:= RGB(0, 0, 189);
    173: Result:= RGB(126, 126, 189);
    174: Result:= RGB(0, 0, 129);
    175: Result:= RGB(86, 86, 129);
    176: Result:= RGB(0, 0, 104);
    177: Result:= RGB(69, 69, 104);
    178: Result:= RGB(0, 0, 79);
    179: Result:= RGB(53, 53, 79);
    180: Result:= RGB(63, 0, 255);
    181: Result:= RGB(191, 170, 255);
    182: Result:= RGB(46, 0, 189);
    183: Result:= RGB(141, 126, 189);
    184: Result:= RGB(31, 0, 129);
    185: Result:= RGB(96, 86, 129);
    186: Result:= RGB(25, 0, 104);
    187: Result:= RGB(78, 69, 104);
    188: Result:= RGB(19, 0, 79);
    189: Result:= RGB(59, 53, 79);
    190: Result:= RGB(127, 0, 255);
    191: Result:= RGB(212, 170, 255);
    192: Result:= RGB(94, 0, 189);
    193: Result:= RGB(157, 126, 189);
    194: Result:= RGB(64, 0, 129);
    195: Result:= RGB(107, 86, 129);
    196: Result:= RGB(52, 0, 104);
    197: Result:= RGB(86, 69, 104);
    198: Result:= RGB(39, 0, 79);
    199: Result:= RGB(66, 53, 79);
    200: Result:= RGB(191, 0, 255);
    201: Result:= RGB(234, 170, 255);
    202: Result:= RGB(141, 0, 189);
    203: Result:= RGB(173, 126, 189);
    204: Result:= RGB(96, 0, 129);
    205: Result:= RGB(118, 86, 129);
    206: Result:= RGB(78, 0, 104);
    207: Result:= RGB(95, 69, 104);
    208: Result:= RGB(59, 0, 79);
    209: Result:= RGB(73, 53, 79);
    210: Result:= RGB(255, 0, 255);
    211: Result:= RGB(255, 170, 255);
    212: Result:= RGB(189, 0, 189);
    213: Result:= RGB(189, 126, 189);
    214: Result:= RGB(129, 0, 129);
    215: Result:= RGB(129, 86, 129);
    216: Result:= RGB(104, 0, 104);
    217: Result:= RGB(104, 69, 104);
    218: Result:= RGB(79, 0, 79);
    219: Result:= RGB(79, 53, 79);
    220: Result:= RGB(255, 0, 191);
    221: Result:= RGB(255, 170, 234);
    222: Result:= RGB(189, 0, 141);
    223: Result:= RGB(189, 126, 173);
    224: Result:= RGB(129, 0, 96);
    225: Result:= RGB(129, 86, 118);
    226: Result:= RGB(104, 0, 78);
    227: Result:= RGB(104, 69, 95);
    228: Result:= RGB(79, 0, 59);
    229: Result:= RGB(79, 53, 73);
    230: Result:= RGB(255, 0, 127);
    231: Result:= RGB(255, 170, 212);
    232: Result:= RGB(189, 0, 94);
    233: Result:= RGB(189, 126, 157);
    234: Result:= RGB(129, 0, 64);
    235: Result:= RGB(129, 86, 107);
    236: Result:= RGB(104, 0, 52);
    237: Result:= RGB(104, 69, 86);
    238: Result:= RGB(79, 0, 39);
    239: Result:= RGB(79, 53, 66);
    240: Result:= RGB(255, 0, 63);
    241: Result:= RGB(255, 170, 191);
    242: Result:= RGB(189, 0, 46);
    243: Result:= RGB(189, 126, 141);
    244: Result:= RGB(129, 0, 31);
    245: Result:= RGB(129, 86, 96);
    246: Result:= RGB(104, 0, 25);
    247: Result:= RGB(104, 69, 78);
    248: Result:= RGB(79, 0, 19);
    249: Result:= RGB(79, 53, 59);
    250: Result:= RGB(51, 51, 51);
    251: Result:= RGB(80, 80, 80);
    252: Result:= RGB(105, 105, 105);
    253: Result:= RGB(130, 130, 130);
    254: Result:= RGB(190, 190, 190);
    255: Result:= RGB(255, 255, 255);
  end;
end;

end.
