unit UfrmTest;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ImgList, AcDvbCommon, AcCommon;

type
  TfrmTest = class(TForm)
    btnOk: TButton;
    cbLayers: TComboBox;
    lblLayers: TLabel;
    imLayers: TImageList;
    procedure FormCreate(Sender: TObject);
    procedure cbLayersDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure FormDestroy(Sender: TObject);
    procedure cbLayersChange(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    function DrawLine: WordBool;
  end;

  function ShowTestDlg: WordBool; stdcall;

implementation

{$R *.dfm}

var
  ActiveLayerIndex: Integer;

function TfrmTest.DrawLine: WordBool;
var
  Space, P1, P2, L: OleVariant;
begin
  Result:= False;
  if VarIsClear(Acad) then Exit;
  try
    P1:= Acad.ActiveDocument.Utility.GetPoint(, '  : ');
    P2:= Acad.ActiveDocument.Utility.GetPoint(P1, '  : ');
    if Acad.ActiveDocument.ActiveSpace = acModelSpace then
      Space:= Acad.ActiveDocument.ModelSpace
    else
      Space:= Acad.ActiveDocument.PaperSpace;
    //     
    L:= Space.AddLine(P1, P2);
    L.Layer:= cbLayers.Text;
    Result:= True;
  except
    //  
  end;
end;

function ShowTestDlg: WordBool;
var
  frmTest: TfrmTest;
begin
  Result:= False;
  frmTest:= TfrmTest.Create(Application);
  try
    if frmTest.ShowModal = mrOk then
      Result:= frmTest.DrawLine;
  finally
    frmTest.Free;
  end;
end;

procedure TfrmTest.FormCreate(Sender: TObject);
begin
  ActiveLayerIndex:= EnumAutoCADLayers(cbLayers.Items);
  if ActiveLayerIndex > -1 then
    cbLayers.ItemIndex:= ActiveLayerIndex;
end;

procedure TfrmTest.cbLayersDrawItem(Control: TWinControl; Index: Integer;
  Rect: TRect; State: TOwnerDrawState);
var
  bmp, tmp: TBitmap;
  n: Integer;
  S: String;
  OldColor: TColor;
  LayerRec: PAcadLayerRec;
  States: TLayerStates;
begin
  with (Control as TComboBox) do
  begin
    LayerRec:= PAcadLayerRec(Items.Objects[Index]);
    Move(LayerRec^.lyState, States, 1);

    S:= Items.Strings[Index];
    bmp:= TBitmap.Create;
    tmp:= TBitmap.Create;
    try
      bmp.Height:= 14; bmp.Width:= 80;
      tmp.Height:= 14; tmp.Width:= 14;

      OldColor:= Canvas.Brush.Color;
      Canvas.Brush.Color:= Color;
      Canvas.FillRect(Rect);
      Canvas.Brush.Color:= OldColor;
      if (lsLayerOn in States) then n:= 0 else n:= 1;
      tmp.Canvas.FillRect(tmp.Canvas.ClipRect);
      imLayers.GetBitmap(n, tmp);
      bmp.Canvas.Draw(0, 0, tmp);

      if not (lsFreeze in States) then n:= 2 else n:= 3;
      tmp.Canvas.FillRect(tmp.Canvas.ClipRect);
      imLayers.GetBitmap(n, tmp);
      bmp.Canvas.Draw(16, 0, tmp);

      if not (lsLock in States) then n:= 4 else n:= 5;
      tmp.Canvas.FillRect(tmp.Canvas.ClipRect);
      imLayers.GetBitmap(n, tmp);
      bmp.Canvas.Draw(32, 0, tmp);

      if (lsPlottable in States) then n:= 6 else n:= 7;
      tmp.Canvas.FillRect(tmp.Canvas.ClipRect);
      imLayers.GetBitmap(n, tmp);
      bmp.Canvas.Draw(48, 0, tmp);

      bmp.Canvas.Brush.Color:= LayerRec.lyColor;
      bmp.Canvas.Rectangle(Classes.Rect(66, 3, 76, 13));

      Canvas.Draw(Rect.Left + 0, Rect.Top, bmp);

      Canvas.FillRect(Classes.Rect(Rect.Left + 78, Rect.Top, Rect.Right, Rect.Bottom));
      if (lsFreeze in States) and not (odFocused in State) then
        Canvas.Font.Color:= clGrayText;
      {else
        Canvas.Font.Color:= Font.Color;}
      Canvas.TextOut(Rect.Left + 80, Rect.Top + 2, S);
      if odFocused in State then
      begin
        Canvas.DrawFocusRect(Rect);
        Canvas.DrawFocusRect(Classes.Rect(Rect.Left + 78, Rect.Top, Rect.Right, Rect.Bottom));
      end;
    finally
      bmp.Free;
      tmp.Free;
    end;
  end;
end;

procedure TfrmTest.FormDestroy(Sender: TObject);
var
  i: Integer;
  LayerRec: PAcadLayerRec;
begin
  with cbLayers do
  begin
    for i:= 0 to Items.Count - 1 do
    begin
      LayerRec:= PAcadLayerRec(Items.Objects[i]);
      Dispose(LayerRec);
    end;
  end;
end;

procedure TfrmTest.cbLayersChange(Sender: TObject);
var
  LayerRec: PAcadLayerRec;
  States: TLayerStates;
begin
  with (Sender as TComboBox) do
  begin
    LayerRec:= PAcadLayerRec(Items.Objects[ItemIndex]);
    Move(LayerRec^.lyState, States, 1);
    if (lsFreeze in States) then
      ItemIndex:= ActiveLayerIndex;
  end;
end;

end.
